/* -*- mode: Java; c-basic-offset: 4 -*- */
/*------------------------------------------------------------------------
 |  FILE            WebPageList.java
 |  PART OF         WebWatcher - A web-page update monitor
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

import java.util.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;

/**
 *
 *
 * @author   Sverre H. Huseby
 *           &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version  $Id: WebPageList.java,v 1.2 2000/09/20 19:39:24 sverrehu Exp $
 */
final class WebPageCanvas
extends Canvas
implements AdjustmentListener, ComponentListener, MouseListener, Observer {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private int               width;
    private int               height;
    private int               maxWidth, maxHeight;
    private WebPageList       parent;
    private WebPageCollection pages;
    private PropertyProvider  main;
    private GUI               gui;
    private WebPage[]         pa;
    private Scrollbar         horiz;
    private Scrollbar         vert;
    private FontMetrics       fontMetr;
    private int[]             col = new int[2];
    private WebPage           selectedPage;

    private void calculateCurrentExtents() {
        int q, w;

        if (fontMetr == null)
            return; /* not yet visible */

        maxHeight = pa.length * fontMetr.getHeight();
        maxWidth = 0;
        for (q = 0; q < pa.length; q++) {
            w = col[col.length - 1] + fontMetr.stringWidth(pa[q].getTitle());
            if (w > maxWidth)
                maxWidth = w;
        }
    }

    private void updateScrollbarLimits() {
        int value, visible, minimum, maximum;

        if (fontMetr == null)
            return; /* not yet visible */

        value = horiz.getValue();
        visible = width;
        minimum = 0;
        maximum = maxWidth;
        horiz.setValues(value, visible, minimum, maximum);
        horiz.setUnitIncrement(fontMetr.charWidth('n'));
        horiz.setBlockIncrement(height);

        value = vert.getValue();
        visible = height;
        minimum = 0;
        maximum = maxHeight;
        vert.setValues(value, visible, minimum, maximum);
        vert.setUnitIncrement(fontMetr.getHeight());
        vert.setBlockIncrement(height);
    }

    private void setSelectedPage(WebPage selectedPage) {
        URL url = null;

        this.selectedPage = selectedPage;
        if (selectedPage != null)
            url = selectedPage.getURL();
        gui.setURLText(url != null ? url.toString() : null);
    }

/*-----------------------------------------------------------------------+
 |  PACKAGE LOCAL PART                                                   |
 +----------------------------------------------------------------------*/
    void refetchInformation() {
        int     q;
        boolean selectionAvailable = false;

        pa = pages.getAll();
        if (selectedPage != null) {
            /* still around? */
            for (q = 0; q < pa.length; q++)
                if (pa[q] == selectedPage) {
                    selectionAvailable = true;
                    break;
                }
            if (!selectionAvailable) {
                setSelectedPage(null);
                gui.setHaveSelection(false);
            }
        }
        if (fontMetr == null)
            return;
        calculateCurrentExtents();
        updateScrollbarLimits();
        repaint();
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public WebPageCanvas(WebPageList parent, PropertyProvider main,
                         WebPageCollection pages,
                         GUI gui, Scrollbar horiz, Scrollbar vert) {
        this.parent = parent;
        this.pages = pages;
        this.main = main;
        this.gui = gui;
        this.horiz = horiz;
        this.vert = vert;

        width = main.getIntProperty("width");
        height = main.getIntProperty("height");
        setSize(width, height);

        pages.addObserver(this);
        pa = pages.getAll();

        horiz.addAdjustmentListener(this);
        vert.addAdjustmentListener(this);

        addComponentListener(this);
        addMouseListener(this);

        setBackground(new Color(0xC0C0C0));
    }

    public WebPage getSelection() {
        return selectedPage;
    }

    public void paint(Graphics g) {
        int     q;
        int     xoffs, yoffs;
        int     x, y;
        int     first, last;
        int     txtWidth;
        String  txt;
        boolean underline;

        if (fontMetr == null) {
            FontMetrics fm;

            setFont(new Font("Helvetica", Font.PLAIN, getFont().getSize()));
            g.setFont(getFont());
            fm = getFontMetrics(getFont());
            col[0] = 5;
            col[1] = col[0] + fm.charWidth('M') + 3;
            fontMetr = fm;
            calculateCurrentExtents();
            updateScrollbarLimits();
        }
        underline = main.getBooleanProperty("underline");
        g.setColor(Color.black);
        g.drawRect(0, 0, width, height);
        xoffs = horiz.getValue();
        yoffs = vert.getValue();
        first = yoffs / fontMetr.getHeight();
        y = fontMetr.getLeading() + fontMetr.getAscent();
        g.translate(-xoffs, -yoffs);
        for (q = 0; q < pa.length; q++) {
            x = col[0];
            switch (pa[q].getStatus()) {
              case WebPage.NO_RESULT:
                g.setColor(Color.black);
                break;
              case WebPage.QUEUED:
                g.setColor(Color.black);
                g.drawString(":", x, y);
                break;
              case WebPage.WORKING:
                g.setColor(Color.black);
                g.drawString("?", x, y);
                break;
              case WebPage.ERROR:
                g.setColor(Color.red);
                g.drawString("X", x, y);
                g.setColor(Color.black);
                break;
              case WebPage.PAGE_UPDATED:
                g.setColor(Color.blue);
                break;
              case WebPage.PAGE_NOT_UPDATED:
                g.setColor(Color.magenta);
                break;
              default:
                g.setColor(Color.black);
                g.drawString("?", x, y);
            }
            x = col[1];
            txt = pa[q].getSomeTitle();
            txtWidth = fontMetr.stringWidth(txt);
            if (pa[q] == selectedPage) {
                Color c = g.getColor();

                g.setColor(Color.yellow.brighter().brighter());
                g.fillRect(x, y - fontMetr.getAscent() - fontMetr.getLeading(),
                           txtWidth, fontMetr.getHeight());
                g.setColor(c);
            }
            g.drawString(txt, x, y);
            if (underline)
                g.drawLine(x, y, x + txtWidth, y);
            y += fontMetr.getHeight();
        }
        g.translate(xoffs, yoffs);
    }

    public Dimension getMinimumSize() {
        return new Dimension(width, height);
    }

    public Dimension getPreferredSize() {
        return getMinimumSize();
    }

    /* AdjustmentListener ***********************************************/
    public void adjustmentValueChanged(AdjustmentEvent e) {
        repaint();
    }

    /* ComponentListener ************************************************/
    public void componentResized(ComponentEvent e) {
        Dimension d;

        d      = getSize();
        width  = d.width;
        height = d.height;
        main.setIntProperty("width", width);
        main.setIntProperty("height", height);
        updateScrollbarLimits();
    }

    public void componentMoved(ComponentEvent e) {
    }

    public void componentShown(ComponentEvent e) {
    }

    public void componentHidden(ComponentEvent e) {
    }

    /* MouseListener ****************************************************/
    public void mouseClicked(MouseEvent e) {
        if (e.isPopupTrigger() && selectedPage != null)
            gui.triggerPopup(this, e.getX(), e.getY());
        if (selectedPage != null && e.getClickCount() > 1)
            gui.doShowPage();
    }

    public void mousePressed(MouseEvent e) {
        int line;

        line = (vert.getValue() + e.getY()) / fontMetr.getHeight();
        setSelectedPage((line < pa.length) ? pa[line] : null);
        repaint();
        gui.setHaveSelection(selectedPage != null);
        if (e.isPopupTrigger() && selectedPage != null)
            gui.triggerPopup(this, e.getX(), e.getY());
    }

    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger() && selectedPage != null)
            gui.triggerPopup(this, e.getX(), e.getY());
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    /* Observer *********************************************************/
    public void update(Observable o, Object arg) {
        refetchInformation();
    }
}

/**
 *
 *
 * @author    Sverre H. Huseby &lt;<A HREF="mailto:shh@thathost.com">
 *                                 shh@thathost.com</A>&gt;
 * @version   $Id: WebPageList.java,v 1.2 2000/09/20 19:39:24 sverrehu Exp $
 */
public final class WebPageList
extends Panel {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private WebPageCollection pages;
    private WebPageCanvas     canvas;
    private Scrollbar         horiz;
    private Scrollbar         vert;

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public WebPageList(WebPageCollection pages, PropertyProvider main,
                       GUI gui) {
        super(new BorderLayout());
        this.pages = pages;
        horiz  = new Scrollbar(Scrollbar.HORIZONTAL);
        vert   = new Scrollbar(Scrollbar.VERTICAL);
        canvas = new WebPageCanvas(this, main, pages, gui, horiz, vert);
        add(canvas, BorderLayout.CENTER);
        add(horiz, BorderLayout.SOUTH);
        add(vert, BorderLayout.EAST);
    }

    public WebPage getSelection() {
        return canvas.getSelection();
    }

    public WebPage[] getAll() {
        return(pages.getAll());
    }

    public void refetchInformation() {
        canvas.refetchInformation();
    }
}
