/* -*- mode: Java; c-basic-offset: 4 -*- */
/*------------------------------------------------------------------------
 |  FILE            GUI.java
 |  PART OF         WebWatcher - A web-page update monitor
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

import no.shhsoft.awt.*;

/**
 *
 *
 * @author   Sverre H. Huseby
 *           &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version  $Id: GUI.java,v 1.1.1.1 2000/10/03 12:41:13 lemire Exp $
 */
public final class GUI
extends Frame
implements WindowListener, ComponentListener, ActionListener {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private WebWatcher main;
    private MenuItem   menuItemMark;
    private MenuItem   menuItemShow;
    private MenuItem   menuItemEdit;
    private MenuItem   menuItemDelete;
    private MenuItem   menuItemRefresh;
    private PopupMenu  popupMenu;
    private Label      urlView;

    private static class MenuLabel {
        final static String WebPage     = "Page";
        final static String New         = "New...";
        final static String Mark        = "Mark as seen";
        final static String Show        = "Browse";
        final static String Edit        = "Edit...";
        final static String Delete      = "Delete";
        final static String Exit        = "Exit";
        final static String View        = "View";
        final static String RefreshAll  = "Refresh all";
        final static String MarkAll     = "Mark all as seen";
        final static String Refresh     = "Refresh";
        final static String Preferences = "Preferences...";
        final static String Help        = "Help";
        final static String Readme      = "README...";
        final static String About       = "About...";
    }

    /* the inner class defined below is not allowed access to the main
     * variable, so we provide a wrapper function for it to call here. */
    private void doQuit() {
        main.doQuit();
    }

    private void doNew() {
        WebPage       page;
        WebPageEditor ed;

        page = new WebPage(main);
        ed = new WebPageEditor(this, page);
        if (ed.isOkPressed())
            main.pages.add(page);
    }

    private void doMark() {
        WebPage page;

        page = winList.getSelection();
        if (page == null)
            return;
        page.markAsSeen();
    }

    private void doMarkAll() {
        WebPage[] pages = winList.getAll();
        for(int k = 0; k < pages.length ; k++)
            pages[k].markAsSeen();

    }

    private void doEdit() {
        WebPage page;

        page = winList.getSelection();
        if (page == null)
            return;
        new WebPageEditor(this, page);
    }

    private void doDelete() {
        WebPage page;

        page = winList.getSelection();
        if (page == null)
            return;
        if (MessageBox.ask(this, "Delete " + page.getSomeTitle())
            == MessageBox.YES)
            main.pages.remove(page);
    }

    private void doRefresh() {
        main.pages.refresh(winList.getSelection());
    }

    private void doRefreshAll() {
        main.pages.refresh();
    }

    private void doPreferences() {
        String         oldPagesFile, newPagesFile;
        PropertyEditor ed;

        oldPagesFile = main.getProperty("pagesFile");
        ed = new PropertyEditor(this, main);
        if (ed.isOkPressed()) {
            newPagesFile = main.getProperty("pagesFile");
            if (newPagesFile.compareTo(oldPagesFile) != 0) {
                /* have a new location for pages file. */
                if ((new File(newPagesFile)).length() > 0L) {
                    if (MessageBox.ask(this,
                                       "Read and merge pages from "
                                       + "the new file?") == MessageBox.YES)
                        try {
                            main.pages.load(newPagesFile, true);
                        } catch (IOException e) {
                            System.err.println("error reading list of "
                                               + "webpages: "
                                               + e.getMessage());
                        }
                }
            }
            /* may have changed sort options, so we better redraw. */
            winList.refetchInformation();
            main.pages.restartUpdater();
        }
    }

    private void doReadme() {
        new ReadmeDialog(this);
    }

    private void doAbout() {
        new AboutDialog(this);
    }

/*-----------------------------------------------------------------------+
 |  PACKAGE LOCAL PART                                                   |
 +----------------------------------------------------------------------*/
    WebPageList winList;

    void doShowPage() {
        WebPage page = winList.getSelection();

        if (page == null)
            return;
        new BrowserLauncher(this, main.getProperty("browser"),
                            main.getProperty("alternatebrowsercommand"), page);
    }

    void setURLText(String text) {
        if (text == null)
            text = "";
        urlView.setText(text);
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public GUI(WebWatcher main) {
        this.main = main;

        addWindowListener(this);
        addComponentListener(this);

        setLocation(main.getIntProperty("screenx"),
                    main.getIntProperty("screeny"));

        /* The menu *************************/
        MenuBar  mb;
        Menu     m;
        MenuItem mi;

        mb = new MenuBar();
        setMenuBar(mb);

        m = new Menu(MenuLabel.WebPage);

        mi = new MenuItem(MenuLabel.New);
        mi.addActionListener(this);
        m.add(mi);

        mi = menuItemMark = new MenuItem(MenuLabel.Mark);
        mi.addActionListener(this);
        m.add(mi);

        mi = new MenuItem(MenuLabel.MarkAll);
        mi.addActionListener(this);
        m.add(mi);

        mi = menuItemShow = new MenuItem(MenuLabel.Show);
        mi.addActionListener(this);
        m.add(mi);

        mi = menuItemEdit = new MenuItem(MenuLabel.Edit);
        mi.addActionListener(this);
        m.add(mi);

        mi = menuItemDelete = new MenuItem(MenuLabel.Delete);
        mi.addActionListener(this);
        m.add(mi);

        m.add(new MenuItem("-"));

        mi = new MenuItem(MenuLabel.Exit);
        mi.addActionListener(this);
        m.add(mi);

        mb.add(m);

        m = new Menu(MenuLabel.View);

        mi = new MenuItem(MenuLabel.RefreshAll);
        mi.addActionListener(this);
        m.add(mi);


        mi = menuItemRefresh = new MenuItem(MenuLabel.Refresh);
        mi.addActionListener(this);
        m.add(mi);

        m.add(new MenuItem("-"));

        mi = new MenuItem(MenuLabel.Preferences);
        mi.addActionListener(this);
        m.add(mi);

        mb.add(m);

        m = new Menu(MenuLabel.Help);

        mi = new MenuItem(MenuLabel.Readme);
        mi.addActionListener(this);
        m.add(mi);

        m.add(new MenuItem("-"));

        mi = new MenuItem(MenuLabel.About);
        mi.addActionListener(this);
        m.add(mi);

        mb.add(m);
        mb.setHelpMenu(m);

        /* popup menu */
        popupMenu = new PopupMenu();
        mi = new MenuItem(MenuLabel.Mark);
        mi.addActionListener(this);
        popupMenu.add(mi);

        mi = new MenuItem(MenuLabel.Show);
        mi.addActionListener(this);
        popupMenu.add(mi);

        mi = new MenuItem(MenuLabel.Edit);
        mi.addActionListener(this);
        popupMenu.add(mi);

        mi = new MenuItem(MenuLabel.Delete);
        mi.addActionListener(this);
        popupMenu.add(mi);

        mi = new MenuItem(MenuLabel.Refresh);
        mi.addActionListener(this);
        popupMenu.add(mi);

        add(popupMenu);

        setHaveSelection(false);

        /* The panes ************************/
        winList = new WebPageList(main.pages, main, this);
        urlView = new Label();
        add(winList, BorderLayout.CENTER);
        add(urlView, BorderLayout.SOUTH);
    }

    public void setHaveSelection(boolean status) {
        menuItemMark.setEnabled(status);
        menuItemShow.setEnabled(status);
        menuItemEdit.setEnabled(status);
        menuItemDelete.setEnabled(status);
        menuItemRefresh.setEnabled(status);
    }

    public void triggerPopup(Component comp, int x, int y) {
        popupMenu.show(comp, x, y);
    }

    /* WindowListener ***************************************************/
    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        doQuit();
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }

    /* ComponentListener ************************************************/
    public void componentResized(ComponentEvent e) {
    }

    public void componentMoved(ComponentEvent e) {
        /* This doesn't work properly nn my Linux box, probably due to
         * a bug in the Java implementation. The location reported is
         * 2 pixels to the left, and 24 pixels above the correct
         * location. */
        Point point;

        if (isVisible()) {
            point = getLocationOnScreen();
            main.setIntProperty("screenx", point.x);
            main.setIntProperty("screeny", point.y);
        }
    }

    public void componentShown(ComponentEvent e) {
    }

    public void componentHidden(ComponentEvent e) {
    }

    /* ActionListener ***************************************************/
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof MenuItem) {
            String label = e.getActionCommand();

            if (label.equals(MenuLabel.New))
                doNew();
            else if (label.equals(MenuLabel.Mark))
                doMark();
            else if (label.equals(MenuLabel.Show))
                doShowPage();
            else if (label.equals(MenuLabel.Edit))
                doEdit();
            else if (label.equals(MenuLabel.Delete))
                doDelete();
            else if (label.equals(MenuLabel.Exit))
                doQuit();
            else if (label.equals(MenuLabel.Refresh))
                doRefresh();
            else if (label.equals(MenuLabel.RefreshAll))
                doRefreshAll();
            else if (label.equals(MenuLabel.MarkAll))
                doMarkAll();
            else if (label.equals(MenuLabel.Preferences))
                doPreferences();
            else if (label.equals(MenuLabel.Readme))
                doReadme();
            else if (label.equals(MenuLabel.About))
                doAbout();
        }
    }
}
