/* -*- mode: Java; c-basic-offset: 4 -*- */
/*------------------------------------------------------------------------
 |  FILE            WebPageCollection.java
 |  PART OF         WebWatcher - A web-page update monitor
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

import java.util.*;
import java.net.*;
import java.io.*;

/**
 *
 *
 * @author   Sverre H. Huseby
 *           &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version  $Id: WebPageCollection.java,v 1.1.1.1 2000/10/03 12:41:14 lemire Exp $
 */
public final class WebPageCollection
extends Observable
implements Observer {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private static final int fileVersion = 1;

    private boolean backupDone = false; /* one backup per session */
    private ArrayList pages;
    private PropertyProvider prop;
    private WebPageCollectionUpdater updater;

    private void stopUpdater() {
        if (updater == null)
            return;
        updater.pleaseStop();
        updater = null;
    }

    private void startUpdater() {
        int interval, left;
        interval = prop.getIntProperty("refreshInterval");
        left = (updater == null) ? interval : updater.minutesLeft();
        if (left > interval)
            left = interval;
        stopUpdater();
        if (interval < 1)
            return;
        updater = new WebPageCollectionUpdater(this, left, interval);
        updater.setDaemon(true);
        updater.setPriority(Thread.MIN_PRIORITY);
        updater.start();
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public WebPageCollection(PropertyProvider prop) {
        pages = new ArrayList();
        this.prop = prop;
        startUpdater();
    }

    public void add(WebPage wp) {
        pages.add(wp);
        wp.addObserver(this);
        setChanged();
        notifyObservers();
        if (prop.getBooleanProperty("refreshNew"))
            updater.refreshPage(wp);
    }

    public void remove(WebPage wp) {
        pages.remove(wp);
        setChanged();
        notifyObservers();
    }

    public WebPage[] getAll() {
        WebPage[] ret = new WebPage[pages.size()];
        pages.toArray(ret);
        if (prop.getBooleanProperty("doSort"))
            WebPage.sortArray(ret, prop.getBooleanProperty("changedFirst"),
                              prop.getBooleanProperty("ignoreCaseInSort"));
        return ret;
    }

    public WebPage getPageForURL(URL url) {
        int     q;
        WebPage ret;

        synchronized (pages) {
            for (q = 0; q < pages.size(); q++) {
                ret = (WebPage) pages.get(q);
                if (ret.getURL().equals(url))
                    return ret;
            }
        }
        return null;
    }

    /* if merging, the pages in memory take precedence over the ones
     * loaded: pages with the same URL will not be read from the
     * file. */
    public void load(String filename, boolean merge)
    throws IOException {
        BufferedReader in = null;
        String         version;
        WebPage        page;

        if (!merge || pages == null)
            pages = new ArrayList();
        stopUpdater();
        try {
            synchronized (pages) {
                in = new BufferedReader(new FileReader(filename));
                /* read header */
                in.readLine();  /* skip description line */
                version = in.readLine();
                if (version == null) {
                    System.err.println("unexpected end of file in URL file");
                    return;
                }
                if (!version.equals("Version: " + fileVersion)) {
                    System.err.println("incompatible file version "
                                       + "for URL file");
                    return;
                }
                /* read entries */
                while (in.ready()) {

                    page = new WebPage(prop);
                    page.load(in);
                    if (!merge || getPageForURL(page.getURL()) == null) {
                        pages.add(page);
                        page.addObserver(this);
                    }
                }

            }
        } catch (FileNotFoundException e) {
            return;
        } finally {
            try {
                in.close();
            } catch(IOException ioe) {
            }
            startUpdater();
            setChanged();
            notifyObservers();
            if (prop.getBooleanProperty("refreshNew"))
                refresh();

        }
    }

    public void load(String filename)
    throws IOException {
        load(filename, false);
    }

    public void save(String filename)
    throws IOException {
        File           theFile;
        File           backupFile;
        BufferedWriter out;
        Iterator       enum;
        WebPage        page;
        String         head = "Datafile for WebWatcher -- do NOT edit!";
        String         version = "Version: " + fileVersion;

        if (!backupDone) {
            theFile = new File(filename);
            backupFile = new File(filename + ".bak");
            backupFile.delete();
            theFile.renameTo(backupFile);
            backupDone = true;
        }
        try {
            out = new BufferedWriter(new FileWriter(filename));
            /* write header */
            out.write(head, 0, head.length());
            out.newLine();
            out.write(version, 0, version.length());
            out.newLine();
            /* write entries */
            for (enum = pages.iterator(); enum.hasNext(); ) {
                page = (WebPage) enum.next();
                page.save(out);
            }
            out.close();
        } catch (FileNotFoundException e) {
            return;
        } finally {
            setChanged();
            notifyObservers();
        }
    }

    public void restartUpdater() {
        startUpdater();
    }

    public void refresh() {
        WebPage page;

        for (Iterator enum = pages.iterator(); enum.hasNext(); ) {
            page = (WebPage) enum.next();
            updater.refreshPage(page);
        }
    }

    public void refresh(WebPage page) {
        if (page == null)
            refresh();
        else
            updater.refreshPage(page);
    }

    /* Observer *********************************************************/
    public void update(Observable o, Object arg) {
        setChanged();
        notifyObservers(o);
    }
}
