/*------------------------------------------------------------------------
 |  FILE            MessageBox.java
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package no.shhsoft.awt;

import java.awt.*;
import java.awt.event.*;

/**
 * A simple helper class for displaying message dialog boxes. The class
 * is not supposed to be instantiated. Instead you call static methods
 * to create and display the message box you want.
 * <P>
 *
 * As seen by this class, a message box is a modal dialog centered on
 * the parent frame. The dialog contains a textual message, with one or
 * more buttons below it.
 * <P>
 *
 * A few methods are available for displaying standardized message boxes
 * such as error and informational messages (with a single "Ok" button
 * in it), and confirmation messages (with "Yes", "No" and "Cancel"
 * buttons). If this is not what you need, you may design your own dialog
 * using the general <CODE>message</CODE> method. With this method,
 * you decide what title and buttons to display. The following buttons
 * are available:
 * <BLOCKQUOTE>
 * "Ok", "Done", "Close", "Yes", "No", "Accept", "Decline",
 * "Apply", "Stop", "Abort", "Cancel" and "Help".
 * </BLOCKQUOTE>
 * Each button text has a matching constant:
 * <BLOCKQUOTE>
 * <CODE>OK</CODE>, <CODE>DONE</CODE>, <CODE>CLOSE</CODE>,
 * <CODE>YES</CODE>, <CODE>NO</CODE>, <CODE>ACCEPT</CODE>,
 * <CODE>DECLINE</CODE>, <CODE>APPLY</CODE>, <CODE>STOP</CODE>,
 * <CODE>ABORT</CODE>, <CODE>CANCEL</CODE> and <CODE>HELP</CODE>.
 * </BLOCKQUOTE>
 * The constants are used when telling <CODE>message</CODE> what buttons
 * to display. Simply combine the button constants using XOR (or use
 * + if bits scare you off). The constants are also used as return values
 * to indicate what button was pressed to get out of the dialog.
 * <P>
 *
 * <B>NOTE:</B> Dialogs are centered on the closest parent
 * <CODE>Frame</CODE> component, even if a non-<CODE>Frame</CODE>
 * component is given as the parent.
 *
 * @author    Sverre H. Huseby &lt;<A HREF="mailto:shh@thathost.com">
 *                                 shh@thathost.com</A>&gt;
 * @version   $Id: MessageBox.java,v 1.1.1.1 2000/10/03 12:41:15 lemire Exp $
 */
public final class MessageBox
extends ModalButtonDialog {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    /* this is the constructor, but you won't get to it! */
    private MessageBox(Component parent, String title, String message,
                       int buttons, int defaultButton) {
        super(parent, title, new Label(message, Label.CENTER),
              buttons, defaultButton);
        setVisible(true);  /* will block, as this box is modal */
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    /**
     * Displays a general dialog box according to the given
     * parameters.
     * <P>
     *
     * The dialog is centered on the nearest <CODE>Frame</CODE>
     * component up in the hierarchy. Does not return untill a button
     * in the dialog is pressed.
     *
     * @param      parent  a parent component
     * @param      title   dialog box title
     * @param      message the message to display
     * @param      buttons XOR'ed button constants of buttons to display
     * @param      defaultButton
     *                     button constant to return when the dialog box
     *                     is closed without pressing any of the buttons
     * @return     A button constant indicating what button was pressed
     *             to close the dialog box.
     */
    public static int message(Component parent,
                              String title, String message,
                              int buttons, int defaultButton) {
        MessageBox mb;

        mb = new MessageBox(parent, title, message, buttons, defaultButton);
        return mb.getButtonCode();
    }

    /**
     * Displays an error message dialog box. The dialog has a single
     * "Ok" button, and the title "Error".
     * <P>
     *
     * The dialog is centered on the nearest <CODE>Frame</CODE>
     * component up in the hierarchy. Does not return untill a button
     * in the dialog is pressed.
     *
     * @param      parent  a parent component
     * @param      message the message to display
     * @return     The constant <CODE>OK</CODE>.
     */
    public static int error(Component parent, String message) {
        return message(parent, "Error", message, OK, OK);
    }

    /**
     * Displays an information dialog box. The dialog has a single
     * "Ok" button, and the title "Information".
     * <P>
     *
     * The dialog is centered on the nearest <CODE>Frame</CODE>
     * component up in the hierarchy. Does not return untill a button
     * in the dialog is pressed.
     *
     * @param      parent  a parent component
     * @param      message the message to display
     * @return     The constant <CODE>OK</CODE>.
     */
    public static int information(Component parent, String message) {
        return message(parent, "Information", message, OK, OK);
    }

    /**
     * Displays a confirmation dialog box. The dialog has the
     * buttons "Yes", "No" and "Cancel", and the title "Confirm".
     * The default button (the one returned if the dialog is closed
     * without pressing any of the buttons) is <CODE>CANCEL</CODE>.
     * <P>
     *
     * The dialog is centered on the nearest <CODE>Frame</CODE>
     * component up in the hierarchy. Does not return untill a button
     * in the dialog is pressed.
     *
     * @param      parent  a parent component
     * @param      message the message to display
     * @return     A button constant indicating what button was pressed
     *             to close the dialog box.
     */
    public static int ask(Component parent, String message) {
        return message(parent, "Confirm", message, YES | NO | CANCEL, CANCEL);
    }
}
